# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "MediaFileErrorEnum",
    },
)


class MediaFileErrorEnum(proto.Message):
    r"""Container for enum describing possible media file errors."""

    class MediaFileError(proto.Enum):
        r"""Enum describing possible media file errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_CREATE_STANDARD_ICON (2):
                Cannot create a standard icon type.
            CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES (3):
                May only select Standard Icons alone.
            CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA (4):
                Image contains both a media file ID and data.
            DUPLICATE_MEDIA (5):
                A media file with given type and reference ID
                already exists.
            EMPTY_FIELD (6):
                A required field was not specified or is an
                empty string.
            RESOURCE_REFERENCED_IN_MULTIPLE_OPS (7):
                A media file may only be modified once per
                call.
            FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE (8):
                Field is not supported for the media sub
                type.
            INVALID_MEDIA_FILE_ID (9):
                The media file ID is invalid.
            INVALID_MEDIA_SUB_TYPE (10):
                The media subtype is invalid.
            INVALID_MEDIA_FILE_TYPE (11):
                The media file type is invalid.
            INVALID_MIME_TYPE (12):
                The mimetype is invalid.
            INVALID_REFERENCE_ID (13):
                The media reference ID is invalid.
            INVALID_YOU_TUBE_ID (14):
                The YouTube video ID is invalid.
            MEDIA_FILE_FAILED_TRANSCODING (15):
                Media file has failed transcoding
            MEDIA_NOT_TRANSCODED (16):
                Media file has not been transcoded.
            MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE (17):
                The media type does not match the actual
                media file's type.
            NO_FIELDS_SPECIFIED (18):
                None of the fields have been specified.
            NULL_REFERENCE_ID_AND_MEDIA_ID (19):
                One of reference ID or media file ID must be
                specified.
            TOO_LONG (20):
                The string has too many characters.
            UNSUPPORTED_TYPE (21):
                The specified type is not supported.
            YOU_TUBE_SERVICE_UNAVAILABLE (22):
                YouTube is unavailable for requesting video
                data.
            YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION (23):
                The YouTube video has a non positive
                duration.
            YOU_TUBE_VIDEO_NOT_FOUND (24):
                The YouTube video ID is syntactically valid
                but the video was not found.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_CREATE_STANDARD_ICON = 2
        CANNOT_SELECT_STANDARD_ICON_WITH_OTHER_TYPES = 3
        CANNOT_SPECIFY_MEDIA_FILE_ID_AND_DATA = 4
        DUPLICATE_MEDIA = 5
        EMPTY_FIELD = 6
        RESOURCE_REFERENCED_IN_MULTIPLE_OPS = 7
        FIELD_NOT_SUPPORTED_FOR_MEDIA_SUB_TYPE = 8
        INVALID_MEDIA_FILE_ID = 9
        INVALID_MEDIA_SUB_TYPE = 10
        INVALID_MEDIA_FILE_TYPE = 11
        INVALID_MIME_TYPE = 12
        INVALID_REFERENCE_ID = 13
        INVALID_YOU_TUBE_ID = 14
        MEDIA_FILE_FAILED_TRANSCODING = 15
        MEDIA_NOT_TRANSCODED = 16
        MEDIA_TYPE_DOES_NOT_MATCH_MEDIA_FILE_TYPE = 17
        NO_FIELDS_SPECIFIED = 18
        NULL_REFERENCE_ID_AND_MEDIA_ID = 19
        TOO_LONG = 20
        UNSUPPORTED_TYPE = 21
        YOU_TUBE_SERVICE_UNAVAILABLE = 22
        YOU_TUBE_VIDEO_HAS_NON_POSITIVE_DURATION = 23
        YOU_TUBE_VIDEO_NOT_FOUND = 24


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from typing import Callable, MutableSequence, Optional, Sequence, Tuple, Union

from google.ads.googleads.v22 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf


try:
    OptionalRetry = Union[
        retries.AsyncRetry, gapic_v1.method._MethodDefault, None
    ]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.ads.googleads.v22.common.types import audience_insights_attribute
from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.enums.types import audience_insights_dimension
from google.ads.googleads.v22.services.types import audience_insights_service
from .transports.base import (
    AudienceInsightsServiceTransport,
    DEFAULT_CLIENT_INFO,
)
from .client import AudienceInsightsServiceClient

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class AudienceInsightsServiceAsyncClient:
    """Audience Insights Service helps users find information about
    groups of people and how they can be reached with Google Ads.
    Accessible to allowlisted customers only.
    """

    _client: AudienceInsightsServiceClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = AudienceInsightsServiceClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = AudienceInsightsServiceClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = (
        AudienceInsightsServiceClient._DEFAULT_ENDPOINT_TEMPLATE
    )
    _DEFAULT_UNIVERSE = AudienceInsightsServiceClient._DEFAULT_UNIVERSE

    common_billing_account_path = staticmethod(
        AudienceInsightsServiceClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        AudienceInsightsServiceClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(
        AudienceInsightsServiceClient.common_folder_path
    )
    parse_common_folder_path = staticmethod(
        AudienceInsightsServiceClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        AudienceInsightsServiceClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        AudienceInsightsServiceClient.parse_common_organization_path
    )
    common_project_path = staticmethod(
        AudienceInsightsServiceClient.common_project_path
    )
    parse_common_project_path = staticmethod(
        AudienceInsightsServiceClient.parse_common_project_path
    )
    common_location_path = staticmethod(
        AudienceInsightsServiceClient.common_location_path
    )
    parse_common_location_path = staticmethod(
        AudienceInsightsServiceClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AudienceInsightsServiceAsyncClient: The constructed client.
        """
        return AudienceInsightsServiceClient.from_service_account_info.__func__(AudienceInsightsServiceAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AudienceInsightsServiceAsyncClient: The constructed client.
        """
        return AudienceInsightsServiceClient.from_service_account_file.__func__(AudienceInsightsServiceAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return AudienceInsightsServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> AudienceInsightsServiceTransport:
        """Returns the transport used by the client instance.

        Returns:
            AudienceInsightsServiceTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = AudienceInsightsServiceClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[
                str,
                AudienceInsightsServiceTransport,
                Callable[..., AudienceInsightsServiceTransport],
            ]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the audience insights service async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,AudienceInsightsServiceTransport,Callable[..., AudienceInsightsServiceTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the AudienceInsightsServiceTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = AudienceInsightsServiceClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.ads.googleads.v22.services.AudienceInsightsServiceAsyncClient`.",
                extra=(
                    {
                        "serviceName": "google.ads.googleads.v22.services.AudienceInsightsService",
                        "universeDomain": getattr(
                            self._client._transport._credentials,
                            "universe_domain",
                            "",
                        ),
                        "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                        "credentialsInfo": getattr(
                            self.transport._credentials,
                            "get_cred_info",
                            lambda: None,
                        )(),
                    }
                    if hasattr(self._client._transport, "_credentials")
                    else {
                        "serviceName": "google.ads.googleads.v22.services.AudienceInsightsService",
                        "credentialsType": None,
                    }
                ),
            )

    async def generate_insights_finder_report(
        self,
        request: Optional[
            Union[
                audience_insights_service.GenerateInsightsFinderReportRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        baseline_audience: Optional[
            audience_insights_service.InsightsAudience
        ] = None,
        specific_audience: Optional[
            audience_insights_service.InsightsAudience
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.GenerateInsightsFinderReportResponse:
        r"""Creates a saved report that can be viewed in the Insights Finder
        tool.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `FieldError <>`__ `HeaderError <>`__
        `InternalError <>`__ `QuotaError <>`__ `RangeError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.GenerateInsightsFinderReportRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.GenerateInsightsFinderReport][google.ads.googleads.v22.services.AudienceInsightsService.GenerateInsightsFinderReport].
            customer_id (:class:`str`):
                Required. The ID of the customer.
                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            baseline_audience (:class:`google.ads.googleads.v22.services.types.InsightsAudience`):
                Required. A baseline audience for
                this report, typically all people in a
                region.

                This corresponds to the ``baseline_audience`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            specific_audience (:class:`google.ads.googleads.v22.services.types.InsightsAudience`):
                Required. The specific audience of
                interest for this report.  The insights
                in the report will be based on
                attributes more prevalent in this
                audience than in the report's baseline
                audience.

                This corresponds to the ``specific_audience`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.GenerateInsightsFinderReportResponse:
                The response message for
                   [AudienceInsightsService.GenerateInsightsFinderReport][google.ads.googleads.v22.services.AudienceInsightsService.GenerateInsightsFinderReport],
                   containing the shareable URL for the report.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, baseline_audience, specific_audience]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.GenerateInsightsFinderReportRequest,
        ):
            request = (
                audience_insights_service.GenerateInsightsFinderReportRequest(
                    request
                )
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if baseline_audience is not None:
            request.baseline_audience = baseline_audience
        if specific_audience is not None:
            request.specific_audience = specific_audience

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_insights_finder_report
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_audience_insights_attributes(
        self,
        request: Optional[
            Union[
                audience_insights_service.ListAudienceInsightsAttributesRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        dimensions: Optional[
            MutableSequence[
                audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
            ]
        ] = None,
        query_text: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.ListAudienceInsightsAttributesResponse:
        r"""Searches for audience attributes that can be used to generate
        insights.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `FieldError <>`__ `HeaderError <>`__
        `InternalError <>`__ `QuotaError <>`__ `RangeError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.ListAudienceInsightsAttributesRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.ListAudienceInsightsAttributes][google.ads.googleads.v22.services.AudienceInsightsService.ListAudienceInsightsAttributes].
            customer_id (:class:`str`):
                Required. The ID of the customer.
                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            dimensions (:class:`MutableSequence[google.ads.googleads.v22.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]`):
                Required. The types of attributes to be returned.
                Supported dimensions are CATEGORY, KNOWLEDGE_GRAPH,
                DEVICE, GEO_TARGET_COUNTRY, SUB_COUNTRY_LOCATION,
                YOUTUBE_LINEUP, AFFINITY_USER_INTEREST,
                IN_MARKET_USER_INTEREST, PARENTAL_STATUS, INCOME_RANGE,
                AGE_RANGE, and GENDER.

                This corresponds to the ``dimensions`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            query_text (:class:`str`):
                Required. A free text query. If the requested dimensions
                include Attributes CATEGORY or KNOWLEDGE_GRAPH, then the
                attributes returned for those dimensions will match or
                be related to this string. For other dimensions, this
                field is ignored and all available attributes are
                returned.

                This corresponds to the ``query_text`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.ListAudienceInsightsAttributesResponse:
                Response message for
                   [AudienceInsightsService.ListAudienceInsightsAttributes][google.ads.googleads.v22.services.AudienceInsightsService.ListAudienceInsightsAttributes].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, dimensions, query_text]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.ListAudienceInsightsAttributesRequest,
        ):
            request = (
                audience_insights_service.ListAudienceInsightsAttributesRequest(
                    request
                )
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if query_text is not None:
            request.query_text = query_text
        if dimensions:
            request.dimensions.extend(dimensions)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_audience_insights_attributes
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_insights_eligible_dates(
        self,
        request: Optional[
            Union[
                audience_insights_service.ListInsightsEligibleDatesRequest, dict
            ]
        ] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.ListInsightsEligibleDatesResponse:
        r"""Lists date ranges for which audience insights data can be
        requested.

        List of thrown errors: `AuthenticationError <>`__
        `AuthorizationError <>`__ `FieldError <>`__ `HeaderError <>`__
        `InternalError <>`__ `QuotaError <>`__ `RangeError <>`__
        `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.ListInsightsEligibleDatesRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.ListInsightsEligibleDates][google.ads.googleads.v22.services.AudienceInsightsService.ListInsightsEligibleDates].
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.ListInsightsEligibleDatesResponse:
                Response message for
                   [AudienceInsightsService.ListInsightsEligibleDates][google.ads.googleads.v22.services.AudienceInsightsService.ListInsightsEligibleDates].

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, audience_insights_service.ListInsightsEligibleDatesRequest
        ):
            request = (
                audience_insights_service.ListInsightsEligibleDatesRequest(
                    request
                )
            )

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.list_insights_eligible_dates
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def generate_audience_composition_insights(
        self,
        request: Optional[
            Union[
                audience_insights_service.GenerateAudienceCompositionInsightsRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        audience: Optional[audience_insights_service.InsightsAudience] = None,
        dimensions: Optional[
            MutableSequence[
                audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
            ]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.GenerateAudienceCompositionInsightsResponse:
        r"""Returns a collection of attributes that are represented in an
        audience of interest, with metrics that compare each attribute's
        share of the audience with its share of a baseline audience.

        List of thrown errors: `AudienceInsightsError <>`__
        `AuthenticationError <>`__ `AuthorizationError <>`__
        `FieldError <>`__ `HeaderError <>`__ `InternalError <>`__
        `QuotaError <>`__ `RangeError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.GenerateAudienceCompositionInsightsRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.GenerateAudienceCompositionInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateAudienceCompositionInsights].
            customer_id (:class:`str`):
                Required. The ID of the customer.
                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            audience (:class:`google.ads.googleads.v22.services.types.InsightsAudience`):
                Required. The audience of interest
                for which insights are being requested.

                This corresponds to the ``audience`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            dimensions (:class:`MutableSequence[google.ads.googleads.v22.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]`):
                Required. The audience dimensions for which composition
                insights should be returned. Supported dimensions are
                KNOWLEDGE_GRAPH, GEO_TARGET_COUNTRY,
                SUB_COUNTRY_LOCATION, YOUTUBE_CHANNEL, YOUTUBE_LINEUP,
                AFFINITY_USER_INTEREST, IN_MARKET_USER_INTEREST,
                PARENTAL_STATUS, INCOME_RANGE, AGE_RANGE, and GENDER.

                This corresponds to the ``dimensions`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.GenerateAudienceCompositionInsightsResponse:
                Response message for
                   [AudienceInsightsService.GenerateAudienceCompositionInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateAudienceCompositionInsights].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, audience, dimensions]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.GenerateAudienceCompositionInsightsRequest,
        ):
            request = audience_insights_service.GenerateAudienceCompositionInsightsRequest(
                request
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if audience is not None:
            request.audience = audience
        if dimensions:
            request.dimensions.extend(dimensions)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_audience_composition_insights
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def generate_suggested_targeting_insights(
        self,
        request: Optional[
            Union[
                audience_insights_service.GenerateSuggestedTargetingInsightsRequest,
                dict,
            ]
        ] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.GenerateSuggestedTargetingInsightsResponse:
        r"""Returns a collection of targeting insights (e.g. targetable
        audiences) that are relevant to the requested audience.

        List of thrown errors: `AudienceInsightsError <>`__
        `AuthenticationError <>`__ `AuthorizationError <>`__
        `FieldError <>`__ `HeaderError <>`__ `InternalError <>`__
        `QuotaError <>`__ `RangeError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.GenerateSuggestedTargetingInsightsRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.GenerateSuggestedTargetingInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateSuggestedTargetingInsights].
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.GenerateSuggestedTargetingInsightsResponse:
                Response message for
                   [AudienceInsightsService.GenerateSuggestedTargetingInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateSuggestedTargetingInsights].

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.GenerateSuggestedTargetingInsightsRequest,
        ):
            request = audience_insights_service.GenerateSuggestedTargetingInsightsRequest(
                request
            )

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_suggested_targeting_insights
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def generate_audience_overlap_insights(
        self,
        request: Optional[
            Union[
                audience_insights_service.GenerateAudienceOverlapInsightsRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        country_location: Optional[criteria.LocationInfo] = None,
        primary_attribute: Optional[
            audience_insights_attribute.AudienceInsightsAttribute
        ] = None,
        dimensions: Optional[
            MutableSequence[
                audience_insights_dimension.AudienceInsightsDimensionEnum.AudienceInsightsDimension
            ]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.GenerateAudienceOverlapInsightsResponse:
        r"""Returns a collection of audience attributes along with estimates
        of the overlap between their potential YouTube reach and that of
        a given input attribute.

        List of thrown errors: `AudienceInsightsError <>`__
        `AuthenticationError <>`__ `AuthorizationError <>`__
        `FieldError <>`__ `HeaderError <>`__ `InternalError <>`__
        `QuotaError <>`__ `RangeError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.GenerateAudienceOverlapInsightsRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.GenerateAudienceOverlapInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateAudienceOverlapInsights].
            customer_id (:class:`str`):
                Required. The ID of the customer.
                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            country_location (:class:`google.ads.googleads.v22.common.types.LocationInfo`):
                Required. The country in which to
                calculate the sizes and overlaps of
                audiences.

                This corresponds to the ``country_location`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            primary_attribute (:class:`google.ads.googleads.v22.common.types.AudienceInsightsAttribute`):
                Required. The audience attribute that
                should be intersected with all other
                eligible audiences.  This must be an
                Affinity or In-Market UserInterest, an
                AgeRange or a Gender.

                This corresponds to the ``primary_attribute`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            dimensions (:class:`MutableSequence[google.ads.googleads.v22.enums.types.AudienceInsightsDimensionEnum.AudienceInsightsDimension]`):
                Required. The types of attributes of which to calculate
                the overlap with the primary_attribute. The values must
                be a subset of AFFINITY_USER_INTEREST,
                IN_MARKET_USER_INTEREST, AGE_RANGE and GENDER.

                This corresponds to the ``dimensions`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.GenerateAudienceOverlapInsightsResponse:
                Response message for
                   [AudienceInsightsService.GenerateAudienceOverlapInsights][google.ads.googleads.v22.services.AudienceInsightsService.GenerateAudienceOverlapInsights].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [
            customer_id,
            country_location,
            primary_attribute,
            dimensions,
        ]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.GenerateAudienceOverlapInsightsRequest,
        ):
            request = audience_insights_service.GenerateAudienceOverlapInsightsRequest(
                request
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if country_location is not None:
            request.country_location = country_location
        if primary_attribute is not None:
            request.primary_attribute = primary_attribute
        if dimensions:
            request.dimensions.extend(dimensions)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_audience_overlap_insights
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def generate_targeting_suggestion_metrics(
        self,
        request: Optional[
            Union[
                audience_insights_service.GenerateTargetingSuggestionMetricsRequest,
                dict,
            ]
        ] = None,
        *,
        customer_id: Optional[str] = None,
        audiences: Optional[
            MutableSequence[audience_insights_service.InsightsAudience]
        ] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> audience_insights_service.GenerateTargetingSuggestionMetricsResponse:
        r"""Returns potential reach metrics for targetable audiences.

        This method helps answer questions like "How many Men aged 18+
        interested in Camping can be reached on YouTube?"

        List of thrown errors: `AudienceInsightsError <>`__
        `AuthenticationError <>`__ `AuthorizationError <>`__
        `FieldError <>`__ `HeaderError <>`__ `InternalError <>`__
        `QuotaError <>`__ `RangeError <>`__ `RequestError <>`__

        Args:
            request (Optional[Union[google.ads.googleads.v22.services.types.GenerateTargetingSuggestionMetricsRequest, dict]]):
                The request object. Request message for
                [AudienceInsightsService.GenerateTargetingSuggestionMetrics][google.ads.googleads.v22.services.AudienceInsightsService.GenerateTargetingSuggestionMetrics].
            customer_id (:class:`str`):
                Required. The ID of the customer.
                This corresponds to the ``customer_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            audiences (:class:`MutableSequence[google.ads.googleads.v22.services.types.InsightsAudience]`):
                Required. Audiences to request
                metrics for.

                This corresponds to the ``audiences`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.ads.googleads.v22.services.types.GenerateTargetingSuggestionMetricsResponse:
                Response message for
                   [AudienceInsightsService.GenerateTargetingSuggestionMetrics][google.ads.googleads.v22.services.AudienceInsightsService.GenerateTargetingSuggestionMetrics].

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [customer_id, audiences]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request,
            audience_insights_service.GenerateTargetingSuggestionMetricsRequest,
        ):
            request = audience_insights_service.GenerateTargetingSuggestionMetricsRequest(
                request
            )

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if customer_id is not None:
            request.customer_id = customer_id
        if audiences:
            request.audiences.extend(audiences)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_targeting_suggestion_metrics
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("customer_id", request.customer_id),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "AudienceInsightsServiceAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("AudienceInsightsServiceAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "DataLinkErrorEnum",
    },
)


class DataLinkErrorEnum(proto.Message):
    r"""Container for enum describing possible DataLink errors."""

    class DataLinkError(proto.Enum):
        r"""Enum describing possible DataLink errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            YOUTUBE_CHANNEL_ID_INVALID (2):
                The requested YouTube Channel ID is invalid.
            YOUTUBE_VIDEO_ID_INVALID (3):
                The requested YouTube Video ID is invalid.
            YOUTUBE_VIDEO_FROM_DIFFERENT_CHANNEL (4):
                The requested YouTube Video ID doesn't belong
                to the requested YouTube Channel ID.
            PERMISSION_DENIED (5):
                A link cannot be created because the customer
                doesn't have the permission.
            INVALID_STATUS (6):
                A link can not be removed or updated because
                the status is invalid.
            INVALID_UPDATE_STATUS (7):
                The input status in the update request is
                invalid.
            INVALID_RESOURCE_NAME (8):
                The input resource name is invalid.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        YOUTUBE_CHANNEL_ID_INVALID = 2
        YOUTUBE_VIDEO_ID_INVALID = 3
        YOUTUBE_VIDEO_FROM_DIFFERENT_CHANNEL = 4
        PERMISSION_DENIED = 5
        INVALID_STATUS = 6
        INVALID_UPDATE_STATUS = 7
        INVALID_RESOURCE_NAME = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v19.common.types import simulation
from google.ads.googleads.v19.enums.types import simulation_modification_method
from google.ads.googleads.v19.enums.types import simulation_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "CampaignSimulation",
    },
)


class CampaignSimulation(proto.Message):
    r"""A campaign simulation. Supported combinations of advertising channel
    type, simulation type and simulation modification method is detailed
    below respectively.

    -  SEARCH - CPC_BID - UNIFORM
    -  SEARCH - CPC_BID - SCALING
    -  SEARCH - TARGET_CPA - UNIFORM
    -  SEARCH - TARGET_CPA - SCALING
    -  SEARCH - TARGET_ROAS - UNIFORM
    -  SEARCH - TARGET_IMPRESSION_SHARE - UNIFORM
    -  SEARCH - BUDGET - UNIFORM
    -  SHOPPING - BUDGET - UNIFORM
    -  SHOPPING - TARGET_ROAS - UNIFORM
    -  MULTI_CHANNEL - TARGET_CPA - UNIFORM
    -  MULTI_CHANNEL - TARGET_ROAS - UNIFORM
    -  DEMAND_GEN - TARGET_CPA - DEFAULT
    -  DISPLAY - TARGET_CPA - UNIFORM
    -  PERFORMANCE_MAX - TARGET_CPA - UNIFORM
    -  PERFORMANCE_MAX - TARGET_ROAS - UNIFORM
    -  PERFORMANCE_MAX - BUDGET - UNIFORM

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the campaign simulation.
            Campaign simulation resource names have the form:

            ``customers/{customer_id}/campaignSimulations/{campaign_id}~{type}~{modification_method}~{start_date}~{end_date}``
        campaign_id (int):
            Output only. Campaign id of the simulation.
        type_ (google.ads.googleads.v19.enums.types.SimulationTypeEnum.SimulationType):
            Output only. The field that the simulation
            modifies.
        modification_method (google.ads.googleads.v19.enums.types.SimulationModificationMethodEnum.SimulationModificationMethod):
            Output only. How the simulation modifies the
            field.
        start_date (str):
            Output only. First day on which the
            simulation is based, in YYYY-MM-DD format.
        end_date (str):
            Output only. Last day on which the simulation
            is based, in YYYY-MM-DD format
        cpc_bid_point_list (google.ads.googleads.v19.common.types.CpcBidSimulationPointList):
            Output only. Simulation points if the simulation type is
            CPC_BID.

            This field is a member of `oneof`_ ``point_list``.
        target_cpa_point_list (google.ads.googleads.v19.common.types.TargetCpaSimulationPointList):
            Output only. Simulation points if the simulation type is
            TARGET_CPA.

            This field is a member of `oneof`_ ``point_list``.
        target_roas_point_list (google.ads.googleads.v19.common.types.TargetRoasSimulationPointList):
            Output only. Simulation points if the simulation type is
            TARGET_ROAS.

            This field is a member of `oneof`_ ``point_list``.
        target_impression_share_point_list (google.ads.googleads.v19.common.types.TargetImpressionShareSimulationPointList):
            Output only. Simulation points if the simulation type is
            TARGET_IMPRESSION_SHARE.

            This field is a member of `oneof`_ ``point_list``.
        budget_point_list (google.ads.googleads.v19.common.types.BudgetSimulationPointList):
            Output only. Simulation points if the
            simulation type is BUDGET.

            This field is a member of `oneof`_ ``point_list``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    type_: simulation_type.SimulationTypeEnum.SimulationType = proto.Field(
        proto.ENUM,
        number=3,
        enum=simulation_type.SimulationTypeEnum.SimulationType,
    )
    modification_method: (
        simulation_modification_method.SimulationModificationMethodEnum.SimulationModificationMethod
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=simulation_modification_method.SimulationModificationMethodEnum.SimulationModificationMethod,
    )
    start_date: str = proto.Field(
        proto.STRING,
        number=5,
    )
    end_date: str = proto.Field(
        proto.STRING,
        number=6,
    )
    cpc_bid_point_list: simulation.CpcBidSimulationPointList = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="point_list",
        message=simulation.CpcBidSimulationPointList,
    )
    target_cpa_point_list: simulation.TargetCpaSimulationPointList = (
        proto.Field(
            proto.MESSAGE,
            number=8,
            oneof="point_list",
            message=simulation.TargetCpaSimulationPointList,
        )
    )
    target_roas_point_list: simulation.TargetRoasSimulationPointList = (
        proto.Field(
            proto.MESSAGE,
            number=9,
            oneof="point_list",
            message=simulation.TargetRoasSimulationPointList,
        )
    )
    target_impression_share_point_list: (
        simulation.TargetImpressionShareSimulationPointList
    ) = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="point_list",
        message=simulation.TargetImpressionShareSimulationPointList,
    )
    budget_point_list: simulation.BudgetSimulationPointList = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="point_list",
        message=simulation.BudgetSimulationPointList,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example creates a billing setup for a customer.

A billing setup is a link between a payments account and a customer. The new
billing setup can either reuse an existing payments account, or create a new
payments account with a given payments profile. Billing setups are applicable
for clients on monthly invoicing only. See here for details about applying for
monthly invoicing: https://support.google.com/google-ads/answer/2375377.
In the case of consolidated billing, a payments account is linked to the
manager account and is linked to a customer account via a billing setup.
"""


import argparse
from datetime import datetime, timedelta
import sys
from typing import Any, Optional
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException


def main(
    client: GoogleAdsClient,
    customer_id: str,
    payments_account_id: Optional[str] = None,
    payments_profile_id: Optional[str] = None,
):
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        payments_account_id: payments account ID to attach to the new billing
            setup. If provided it must be formatted as "1234-5678-9012-3456".
        payments_profile_id: payments profile ID to attach to a new payments
            account and to the new billing setup. If provided it must be
            formatted as "1234-5678-9012".
    """
    billing_setup = create_billing_setup(
        client, customer_id, payments_account_id, payments_profile_id
    )
    set_billing_setup_date_times(client, customer_id, billing_setup)
    billing_setup_operation = client.get_type("BillingSetupOperation")
    client.copy_from(billing_setup_operation.create, billing_setup)
    billing_setup_service = client.get_service("BillingSetupService")
    response = billing_setup_service.mutate_billing_setup(
        customer_id=customer_id, operation=billing_setup_operation
    )
    print(
        "Added new billing setup with resource name "
        f"{response.result.resource_name}"
    )


def create_billing_setup(
    client: GoogleAdsClient,
    customer_id: str,
    payments_account_id: Optional[str] = None,
    payments_profile_id: Optional[str] = None,
) -> Any:
    """Creates and returns a new billing setup instance.

    The new billing setup will have its payment details populated. One of the
    payments_account_id or payments_profile_id must be provided.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        payments_account_id: payments account ID to attach to the new billing
            setup. If provided it must be formatted as "1234-5678-9012-3456".
        payments_profile_id: payments profile ID to attach to a new payments
            account and to the new billing setup. If provided it must be
            formatted as "1234-5678-9012".

    Returns:
        A newly created BillingSetup instance.
    """
    billing_setup = client.get_type("BillingSetup")

    # Sets the appropriate payments account field.
    if payments_account_id != None:
        # If a payments account ID has been provided, set the payments_account
        # field to the full resource name of the given payments account ID.
        # You can list available payments accounts via the
        # PaymentsAccountService's ListPaymentsAccounts method.
        billing_setup.payments_account = client.get_service(
            "BillingSetupService"
        ).payments_account_path(customer_id, payments_account_id)
    elif payments_profile_id != None:
        # Otherwise, create a new payments account by setting the
        # payments_account_info field
        # See https://support.google.com/google-ads/answer/7268503
        # for more information about payments profiles.
        billing_setup.payments_account_info.payments_account_name = (
            f"Payments Account #{uuid4()}"
        )
        billing_setup.payments_account_info.payments_profile_id = (
            payments_profile_id
        )

    return billing_setup


def set_billing_setup_date_times(
    client: GoogleAdsClient, customer_id: str, billing_setup: Any
):
    """Sets the starting and ending date times for the new billing setup.

    Queries the customer's account to see if there are any approved billing
    setups. If there are any, the new billing setup starting date time is set to
    one day after the last. If not, the billing setup is set to start
    immediately. The ending date is set to one day after the starting date time.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        billing_setup: the billing setup whose starting and ending date times
            will be set.
    """
    # The query to search existing approved billing setups in the end date time
    # descending order. See get_billing_setup.py for a more detailed example of
    # how to retrieve billing setups.
    query = """
      SELECT
        billing_setup.end_date_time
      FROM billing_setup
      WHERE billing_setup.status = APPROVED
      ORDER BY billing_setup.end_date_time DESC
      LIMIT 1"""

    ga_service = client.get_service("GoogleAdsService")
    stream = ga_service.search_stream(customer_id=customer_id, query=query)
    # Coercing the response iterator to a list causes the stream to be fully
    # consumed so that we can easily access the last row in the request.
    batches = list(stream)
    # Checks if any results were included in the response.
    if batches:
        # Retrieves the ending_date_time of the last BillingSetup.
        last_batch = batches[0]
        last_row = last_batch.results[0]
        last_ending_date_time = last_row.billing_setup.end_date_time

        if not last_ending_date_time:
            # A null ending date time indicates that the current billing setup
            # is set to run indefinitely. Billing setups cannot overlap, so
            # throw an exception in this case.
            raise Exception(
                "Cannot set starting and ending date times for the new billing "
                "setup; the latest existing billing setup is set to run "
                "indefinitely."
            )

        try:
            # BillingSetup.end_date_time is a string that can be in the format
            # %Y-%m-%d or %Y-%m-%d %H:%M:%S. This checks for the first format.
            end_date_time_obj = datetime.strptime(
                last_ending_date_time, "%Y-%m-%d"
            )
        except ValueError:
            # If a ValueError is raised then the end_date_time string is in the
            # second format that includes hours, minutes and seconds.
            end_date_time_obj = datetime.strptime(
                last_ending_date_time, "%Y-%m-%d %H:%M:%S"
            )

        # Sets the new billing setup start date to one day after the end date.
        start_date = end_date_time_obj + timedelta(days=1)
    else:
        # If there are no BillingSetup objects to retrieve, the only acceptable
        # start date time is today.
        start_date = datetime.now()

    billing_setup.start_date_time = start_date.strftime("%Y-%m-%d %H:%M:%S")
    billing_setup.end_date_time = (start_date + timedelta(days=1)).strftime(
        "%Y-%m-%d %H:%M:%S"
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=("Creates a billing setup for a given customer.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    # Creates a mutually exclusive argument group to ensure that only one of the
    # following two arguments are given, otherwise it will raise an error.
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "-a",
        "--payments_account_id",
        type=str,
        help="Either a payments account ID or a payments profile ID must be "
        "provided for the example to run successfully. "
        "See: https://developers.google.com/google-ads/api/docs/billing/billing-setups#creating_new_billing_setups. "
        "Provide an existing payments account ID to link to the new "
        "billing setup. Must be formatted as '1234-5678-9012-3456'.",
    )
    group.add_argument(
        "-p",
        "--payments_profile_id",
        type=str,
        help="Either a payments account ID or a payments profile ID must be "
        "provided for the example to run successfully. "
        "See: https://developers.google.com/google-ads/api/docs/billing/billing-setups#creating_new_billing_setups. "
        "Provide an existing payments profile ID to link to a new payments "
        "account and the new billing setup. Must be formatted as: "
        "'1234-5678-9012-3456'.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client,
            args.customer_id,
            args.payments_account_id,
            args.payments_profile_id,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import consent_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "Consent",
    },
)


class Consent(proto.Message):
    r"""Consent

    Attributes:
        ad_user_data (google.ads.googleads.v21.enums.types.ConsentStatusEnum.ConsentStatus):
            This represents consent for ad user data.
        ad_personalization (google.ads.googleads.v21.enums.types.ConsentStatusEnum.ConsentStatus):
            This represents consent for ad
            personalization. This can only be set for
            OfflineUserDataJobService and UserDataService.
    """

    ad_user_data: consent_status.ConsentStatusEnum.ConsentStatus = proto.Field(
        proto.ENUM,
        number=1,
        enum=consent_status.ConsentStatusEnum.ConsentStatus,
    )
    ad_personalization: consent_status.ConsentStatusEnum.ConsentStatus = (
        proto.Field(
            proto.ENUM,
            number=2,
            enum=consent_status.ConsentStatusEnum.ConsentStatus,
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Updates the audience target restriction of a given ad group to bid only."""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.targeting_setting import (
    TargetingSetting,
    TargetRestriction,
)
from google.ads.googleads.v22.enums.types.targeting_dimension import (
    TargetingDimensionEnum,
)
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsResponse,
)
from google.api_core import protobuf_helpers


def main(client: GoogleAdsClient, customer_id: str, ad_group_id: str) -> None:
    """Updates the audience target restriction of a given ad group to bid only.

    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id: The client customer ID string.
        ad_group_id: The ad group ID for which to update the audience targeting
            restriction.
    """
    # Get the GoogleAdsService client.
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Create a search request that retrieves the targeting settings from a given
    # ad group.
    # [START update_audience_target_restriction]
    query: str = f"""
        SELECT
          ad_group.id,
          ad_group.name,
          ad_group.targeting_setting.target_restrictions
        FROM ad_group
        WHERE ad_group.id = {ad_group_id}"""
    # [END update_audience_target_restriction]

    # Issue the search request.
    search_response: SearchGoogleAdsResponse = googleads_service.search(
        customer_id=customer_id, query=query
    )

    targeting_dimension_enum: TargetingDimensionEnum = (
        client.enums.TargetingDimensionEnum
    )

    # Create an empty TargetingSetting instance.
    targeting_setting: TargetingSetting = client.get_type("TargetingSetting")

    # Create a flag that specifies whether or not we should update the
    # targeting setting. We should only do this if we find an audience
    # target restriction with bid_only set to false.
    should_update_targeting_setting: bool = False

    ad_group: AdGroup = next(iter(search_response)).ad_group

    print(
        f"Ad group with ID {ad_group.id} and name '{ad_group.name}' "
        "was found with the following targeting restrictions:"
    )

    target_restrictions: "list[TargetRestriction]" = (
        ad_group.targeting_setting.target_restrictions
    )

    # Loop through and print each of the target restrictions.
    # Reconstruct the TargetingSetting object with the updated audience
    # target restriction because Google Ads will overwrite the entire
    # targeting_setting field of the ad group when the field mask
    # includes targeting_setting in an update operation.
    # [START update_audience_target_restriction_1]
    target_restriction: TargetRestriction
    for target_restriction in target_restrictions:
        targeting_dimension: TargetingDimensionEnum.TargetingDimension = (
            target_restriction.targeting_dimension
        )
        bid_only: bool = target_restriction.bid_only

        print(
            "\tTargeting restriction with targeting dimension "
            f"'{targeting_dimension.name}' "
            f"and bid only set to '{bid_only}'."
        )

        # Add the target restriction to the TargetingSetting object as
        # is if the targeting dimension has a value other than audience
        # because those should not change.
        if targeting_dimension != targeting_dimension_enum.AUDIENCE:
            targeting_setting.target_restrictions.append(target_restriction)
        elif not bid_only:
            should_update_targeting_setting: bool = True

            # Add an audience target restriction with bid_only set to
            # true to the targeting setting object. This has the effect
            # of setting the audience target restriction to
            # "Observation". For more details about the targeting
            # setting, visit
            # https://support.google.com/google-ads/answer/7365594.
            new_target_restriction: TargetRestriction = (
                targeting_setting.target_restrictions.add()
            )
            new_target_restriction.targeting_dimension = (
                targeting_dimension_enum.AUDIENCE
            )
            new_target_restriction.bid_only = True
            # [END update_audience_target_restriction_1]

    # Only update the TargetSetting on the ad group if there is an audience
    # TargetRestriction with bid_only set to false.
    if should_update_targeting_setting:
        update_targeting_setting(
            client, customer_id, ad_group_id, targeting_setting
        )
    else:
        print("No target restrictions to update.")


# [START update_audience_target_restriction_2]
def update_targeting_setting(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    targeting_setting: TargetingSetting,
) -> None:
    """Updates the given TargetingSetting of an ad group.

    Args:
        client: The Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID for which to update the audience targeting
            restriction.
        targeting_setting: The updated targeting setting.
    """
    # Get the AdGroupService client.
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )

    # Construct an operation that will update the ad group.
    ad_group_operation: AdGroupOperation = client.get_type("AdGroupOperation")

    # Populate the ad group object with the updated targeting setting.
    ad_group: AdGroup = ad_group_operation.update
    ad_group.resource_name = ad_group_service.ad_group_path(
        customer_id, ad_group_id
    )
    ad_group.targeting_setting.target_restrictions.extend(
        targeting_setting.target_restrictions
    )
    # Use the field_mask utility to derive the update mask. This mask tells the
    # Google Ads API which attributes of the ad group you want to change.
    client.copy_from(
        ad_group_operation.update_mask,
        protobuf_helpers.field_mask(None, ad_group._pb),
    )

    # Send the operation in a mutate request and print the resource name of the
    # updated object.
    mutate_ad_groups_response: MutateAdGroupsResponse = (
        ad_group_service.mutate_ad_groups(
            customer_id=customer_id, operations=[ad_group_operation]
        )
    )
    print(
        "Updated targeting setting of ad group with resource name "
        f"'{mutate_ad_groups_response.results[0].resource_name}'; set the "
        "audience target restriction to 'Observation'."
    )
    # [END update_audience_target_restriction_2]


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Updates the audience target restriction of a given ad "
        "group to bid only."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="The ad group ID for which to update the audience targeting "
        "restriction.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.ad_group_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import (
    criterion_category_availability,
)
from google.ads.googleads.v22.enums.types import user_interest_taxonomy_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "UserInterest",
    },
)


class UserInterest(proto.Message):
    r"""A user interest: a particular interest-based vertical to be
    targeted.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the user interest. User
            interest resource names have the form:

            ``customers/{customer_id}/userInterests/{user_interest_id}``
        taxonomy_type (google.ads.googleads.v22.enums.types.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType):
            Output only. Taxonomy type of the user
            interest.
        user_interest_id (int):
            Output only. The ID of the user interest.

            This field is a member of `oneof`_ ``_user_interest_id``.
        name (str):
            Output only. The name of the user interest.

            This field is a member of `oneof`_ ``_name``.
        user_interest_parent (str):
            Output only. The parent of the user interest.

            This field is a member of `oneof`_ ``_user_interest_parent``.
        launched_to_all (bool):
            Output only. True if the user interest is
            launched to all channels and locales.

            This field is a member of `oneof`_ ``_launched_to_all``.
        availabilities (MutableSequence[google.ads.googleads.v22.common.types.CriterionCategoryAvailability]):
            Output only. Availability information of the
            user interest.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    taxonomy_type: (
        user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=user_interest_taxonomy_type.UserInterestTaxonomyTypeEnum.UserInterestTaxonomyType,
    )
    user_interest_id: int = proto.Field(
        proto.INT64,
        number=8,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    user_interest_parent: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    launched_to_all: bool = proto.Field(
        proto.BOOL,
        number=11,
        optional=True,
    )
    availabilities: MutableSequence[
        criterion_category_availability.CriterionCategoryAvailability
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message=criterion_category_availability.CriterionCategoryAvailability,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
#
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shows how to write data from a basic campaign stats report to a CSV file.

This is important for migration because reports from the AdWords API are
returned in CSV format, so clients that are migrating may want to temporarily
convert Google Ads API reports to CSV while migrating their applications.

Examples:
    Write to file output.csv in the same directory as script with headers.
        $ python get_campaign_stats_to_csv.py -c 0123456789 -o output.csv -w

    Write to file output.csv in the same directory as script without headers.
        $ python get_campaign_stats_to_csv.py -c 0123456789 -o output.csv
"""
import argparse
import csv
from collections.abc import Iterator
import os
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
    SearchGoogleAdsStreamRequest,
    SearchGoogleAdsStreamResponse,
)


_DEFAULT_FILE_NAME = "campaign_report_to_csv_results.csv"
_QUERY: str = """
  SELECT
      customer.descriptive_name,
      segments.date,
      campaign.name,
      campaign.contains_eu_political_advertising,
      metrics.impressions,
      metrics.clicks,
      metrics.cost_micros
  FROM campaign
  WHERE
      segments.date DURING LAST_7_DAYS
  ORDER BY metrics.impressions DESC
  LIMIT 25"""


def main(
    client: GoogleAdsClient,
    customer_id: str,
    output_file: str,
    write_headers: bool,
) -> None:
    """Writes rows returned from a search_stream request to a CSV file.
    Args:
        client: An initialized GoogleAdsClient instance.
        customer_id (str): The client customer ID string.
        output_file (str): Filename of the file to write the report data to.
        write_headers (bool): From argparse, True if arg is provided.
    """
    file_dir: str = os.path.dirname(os.path.abspath(__file__))
    file_path: str = os.path.join(file_dir, output_file)
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")

    # Issues a search request using streaming.
    search_request: SearchGoogleAdsStreamRequest = client.get_type(
        "SearchGoogleAdsStreamRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = _QUERY
    stream: Iterator[SearchGoogleAdsStreamResponse] = ga_service.search_stream(
        search_request
    )

    with open(file_path, "w", newline="") as f:
        writer = csv.writer(f)

        # Define a list of headers for the first row.
        headers: list[str] = [
            "Account",
            "Date",
            "Campaign",
            "Contains EU Political Advertising",
            "Impressions",
            "Clicks",
            "Cost",
        ]

        # If the write_headers flag was passed, write header row to the CSV
        if write_headers:
            writer.writerow(headers)

        batch: SearchGoogleAdsStreamResponse
        for batch in stream:
            row: GoogleAdsRow
            for row in batch.results:
                # Use the CSV writer to write the individual GoogleAdsRow
                # fields returned in the SearchGoogleAdsStreamResponse.
                writer.writerow(
                    [
                        row.customer.descriptive_name,
                        row.segments.date,
                        row.campaign.name,
                        row.campaign.contains_eu_political_advertising.name,
                        row.metrics.impressions,
                        row.metrics.clicks,
                        row.metrics.cost_micros,
                    ]
                )

        print(f"Customer {customer_id} report written to {output_file}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Retrieves a campaign stats and writes to CSV file."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help=(
            "The Google Ads customer ID of the account you would like to get "
            "the report for to write to CSV."
        ),
    )
    parser.add_argument(
        "-o",
        "--output_file",
        type=str,
        required=False,
        default=_DEFAULT_FILE_NAME,
        help=(
            "Name of the local CSV file to save the report to. File will be "
            "saved in the same directory as the script."
        ),
    )
    # Optional boolean argument for writing headers.
    parser.add_argument(
        "-w",
        "--write_headers",
        action="store_true",
        help=(
            "Writes headers to the CSV file if argument is supplied. Simply "
            "add -w if you want the headers defined in the script to be "
            "added as the first row in the CSV file."
        ),
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(
            googleads_client,
            args.customer_id,
            args.output_file,
            args.write_headers,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AssetFieldTypeEnum",
    },
)


class AssetFieldTypeEnum(proto.Message):
    r"""Container for enum describing the possible placements of an
    asset.

    """

    class AssetFieldType(proto.Enum):
        r"""Enum describing the possible placements of an asset.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            HEADLINE (2):
                The asset is linked for use as a headline.
            DESCRIPTION (3):
                The asset is linked for use as a description.
            MANDATORY_AD_TEXT (4):
                The asset is linked for use as mandatory ad
                text.
            MARKETING_IMAGE (5):
                The asset is linked for use as a marketing
                image.
            MEDIA_BUNDLE (6):
                The asset is linked for use as a media
                bundle.
            YOUTUBE_VIDEO (7):
                The asset is linked for use as a YouTube
                video.
            BOOK_ON_GOOGLE (8):
                The asset is linked to indicate that a hotels
                campaign is "Book on Google" enabled.
            LEAD_FORM (9):
                The asset is linked for use as a Lead Form
                extension.
            PROMOTION (10):
                The asset is linked for use as a Promotion
                extension.
            CALLOUT (11):
                The asset is linked for use as a Callout
                extension.
            STRUCTURED_SNIPPET (12):
                The asset is linked for use as a Structured
                Snippet extension.
            SITELINK (13):
                The asset is linked for use as a Sitelink.
            MOBILE_APP (14):
                The asset is linked for use as a Mobile App
                extension.
            HOTEL_CALLOUT (15):
                The asset is linked for use as a Hotel
                Callout extension.
            CALL (16):
                The asset is linked for use as a Call
                extension.
            PRICE (24):
                The asset is linked for use as a Price
                extension.
            LONG_HEADLINE (17):
                The asset is linked for use as a long
                headline.
            BUSINESS_NAME (18):
                The asset is linked for use as a business
                name.
            SQUARE_MARKETING_IMAGE (19):
                The asset is linked for use as a square
                marketing image.
            PORTRAIT_MARKETING_IMAGE (20):
                The asset is linked for use as a portrait
                marketing image.
            LOGO (21):
                The asset is linked for use as a logo.
            LANDSCAPE_LOGO (22):
                The asset is linked for use as a landscape
                logo.
            VIDEO (23):
                The asset is linked for use as a non YouTube
                logo.
            CALL_TO_ACTION_SELECTION (25):
                The asset is linked for use to select a
                call-to-action.
            AD_IMAGE (26):
                The asset is linked for use to select an ad
                image.
            BUSINESS_LOGO (27):
                The asset is linked for use as a business
                logo.
            HOTEL_PROPERTY (28):
                The asset is linked for use as a hotel
                property in a Performance Max for travel goals
                campaign.
            DEMAND_GEN_CAROUSEL_CARD (30):
                The asset is linked for use as a Demand Gen
                carousel card.
            BUSINESS_MESSAGE (31):
                The asset is linked for use as a Business
                Message.
            TALL_PORTRAIT_MARKETING_IMAGE (32):
                The asset is linked for use as a tall
                portrait marketing image.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        HEADLINE = 2
        DESCRIPTION = 3
        MANDATORY_AD_TEXT = 4
        MARKETING_IMAGE = 5
        MEDIA_BUNDLE = 6
        YOUTUBE_VIDEO = 7
        BOOK_ON_GOOGLE = 8
        LEAD_FORM = 9
        PROMOTION = 10
        CALLOUT = 11
        STRUCTURED_SNIPPET = 12
        SITELINK = 13
        MOBILE_APP = 14
        HOTEL_CALLOUT = 15
        CALL = 16
        PRICE = 24
        LONG_HEADLINE = 17
        BUSINESS_NAME = 18
        SQUARE_MARKETING_IMAGE = 19
        PORTRAIT_MARKETING_IMAGE = 20
        LOGO = 21
        LANDSCAPE_LOGO = 22
        VIDEO = 23
        CALL_TO_ACTION_SELECTION = 25
        AD_IMAGE = 26
        BUSINESS_LOGO = 27
        HOTEL_PROPERTY = 28
        DEMAND_GEN_CAROUSEL_CARD = 30
        BUSINESS_MESSAGE = 31
        TALL_PORTRAIT_MARKETING_IMAGE = 32


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import identity_verification_program
from google.ads.googleads.v20.enums.types import (
    identity_verification_program_status,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "StartIdentityVerificationRequest",
        "GetIdentityVerificationRequest",
        "GetIdentityVerificationResponse",
        "IdentityVerification",
        "IdentityVerificationProgress",
        "IdentityVerificationRequirement",
    },
)


class StartIdentityVerificationRequest(proto.Message):
    r"""Request message for
    [IdentityVerificationService.StartIdentityVerification].

    Attributes:
        customer_id (str):
            Required. The Id of the customer for whom we
            are creating this verification.
        verification_program (google.ads.googleads.v20.enums.types.IdentityVerificationProgramEnum.IdentityVerificationProgram):
            Required. The verification program type for
            which we want to start the verification.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    verification_program: (
        identity_verification_program.IdentityVerificationProgramEnum.IdentityVerificationProgram
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=identity_verification_program.IdentityVerificationProgramEnum.IdentityVerificationProgram,
    )


class GetIdentityVerificationRequest(proto.Message):
    r"""Request message for
    [IdentityVerificationService.GetIdentityVerification].

    Attributes:
        customer_id (str):
            Required.  The ID of the customer for whom we
            are requesting verification  information.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class GetIdentityVerificationResponse(proto.Message):
    r"""Response message for
    [IdentityVerificationService.GetIdentityVerification].

    Attributes:
        identity_verification (MutableSequence[google.ads.googleads.v20.services.types.IdentityVerification]):
            List of identity verifications for the
            customer.
    """

    identity_verification: MutableSequence["IdentityVerification"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="IdentityVerification",
        )
    )


class IdentityVerification(proto.Message):
    r"""An identity verification for a customer.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        verification_program (google.ads.googleads.v20.enums.types.IdentityVerificationProgramEnum.IdentityVerificationProgram):
            The verification program type.
        identity_verification_requirement (google.ads.googleads.v20.services.types.IdentityVerificationRequirement):
            The verification requirement for this
            verification program for this customer.

            This field is a member of `oneof`_ ``_identity_verification_requirement``.
        verification_progress (google.ads.googleads.v20.services.types.IdentityVerificationProgress):
            Information regarding progress for this
            verification program for this customer.

            This field is a member of `oneof`_ ``_verification_progress``.
    """

    verification_program: (
        identity_verification_program.IdentityVerificationProgramEnum.IdentityVerificationProgram
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=identity_verification_program.IdentityVerificationProgramEnum.IdentityVerificationProgram,
    )
    identity_verification_requirement: "IdentityVerificationRequirement" = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            optional=True,
            message="IdentityVerificationRequirement",
        )
    )
    verification_progress: "IdentityVerificationProgress" = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message="IdentityVerificationProgress",
    )


class IdentityVerificationProgress(proto.Message):
    r"""Information regarding the verification progress for a
    verification program type.

    Attributes:
        program_status (google.ads.googleads.v20.enums.types.IdentityVerificationProgramStatusEnum.IdentityVerificationProgramStatus):
            Current Status (PENDING_USER_ACTION, SUCCESS, FAILURE etc)
        invitation_link_expiration_time (str):
            The timestamp when the action url will expire
            in "yyyy-MM-dd HH:mm:ss" format.
        action_url (str):
            Action URL for user to complete verification
            for the given verification program type.
    """

    program_status: (
        identity_verification_program_status.IdentityVerificationProgramStatusEnum.IdentityVerificationProgramStatus
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=identity_verification_program_status.IdentityVerificationProgramStatusEnum.IdentityVerificationProgramStatus,
    )
    invitation_link_expiration_time: str = proto.Field(
        proto.STRING,
        number=2,
    )
    action_url: str = proto.Field(
        proto.STRING,
        number=3,
    )


class IdentityVerificationRequirement(proto.Message):
    r"""Information regarding the verification requirement for a
    verification program type.

    Attributes:
        verification_start_deadline_time (str):
            The deadline to start verification in
            "yyyy-MM-dd HH:mm:ss" format.
        verification_completion_deadline_time (str):
            The deadline to submit verification.
    """

    verification_start_deadline_time: str = proto.Field(
        proto.STRING,
        number=1,
    )
    verification_completion_deadline_time: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "AdNetworkTypeEnum",
    },
)


class AdNetworkTypeEnum(proto.Message):
    r"""Container for enumeration of Google Ads network types."""

    class AdNetworkType(proto.Enum):
        r"""Enumerates Google Ads network types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            SEARCH (2):
                Google search.
            SEARCH_PARTNERS (3):
                Search partners.
            CONTENT (4):
                Display Network.
            MIXED (7):
                Cross-network.
            YOUTUBE (8):
                YouTube
            GOOGLE_TV (9):
                Google TV
            GOOGLE_OWNED_CHANNELS (10):
                This network is used for Google Owned
                channels such as Discover feed, Gmail, YouTube.
                Starting with V20, Demand Gen Stats will be
                attributed to more granular network types such
                as GMAIL, DISCOVER, MAPS, YOUTUBE; this value
                will only be used for historical data.
            GMAIL (11):
                Gmail
            DISCOVER (12):
                Discover Feed
            MAPS (13):
                Maps
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        SEARCH = 2
        SEARCH_PARTNERS = 3
        CONTENT = 4
        MIXED = 7
        YOUTUBE = 8
        GOOGLE_TV = 9
        GOOGLE_OWNED_CHANNELS = 10
        GMAIL = 11
        DISCOVER = 12
        MAPS = 13


__all__ = tuple(sorted(__protobuf__.manifest))
